import java.awt.Image;
import java.awt.Toolkit;
import java.awt.image.ColorModel;
import java.awt.image.MemoryImageSource;
import java.awt.image.PixelGrabber;

import utilities.log.Log;

// Don't try writing to JPEGs.  It will have no effect...
public class ImageWrapper {
  private Image image;
  private int[] source;
  private int width;
  private int height;

  private static Toolkit toolkit;

  static {
    toolkit = Toolkit.getDefaultToolkit();
  }

  // Constructor...
  public ImageWrapper(final ImageWrapper image) {
    width = image.getWidth(null);
    height = image.getHeight(null);
    int[] in_pix = image.getSource();
    int[] out_pix = new int[in_pix.length];
    for (int i = 0; i < width; i++) {
      for (int j = 0; j < height; j++) {
        out_pix[i + width * j] = in_pix[i + width * j];
      }
    }

    createImageFromArray(out_pix, width, height);
  }

  // duplicate existing

  public ImageWrapper(final Image image) {
    setImage(image);
  }

  public ImageWrapper(final int[] a, final int w, final int h, final boolean x) {
    image = toolkit.createImage(new MemoryImageSource(w, h, a, 0, w));
  }

  public ImageWrapper(final int[] a, final int w, final int h) {
    createImageFromArray(a, w, h);
  }

  public final void createImageFromArray(final int[] a, final int w, final int h) {
    this.source = a;
    this.width = w;
    this.height = h;
    this.image = toolkit.createImage(new MemoryImageSource(w, h, ColorModel.getRGBdefault(), a, 0, w));
  }

  // used for JPEGs...
  public final void setImage(final Image image) {
    this.image = image;
    this.width = image.getWidth(null);
    this.height = image.getHeight(null);
    this.source = null;
  }

  public final Image getImage() {
    return image;
  }

  //  public final int getWidth() {
  //    return width;
  //  }
  //
  //  public final int getHeight() {
  //    return height;
  //  }

  public final int getWidth(final Object o) {
    return image.getWidth(null);
  }

  public final int getHeight(final Object o) {
    return image.getHeight(null);
  }

  public final int[] getSource() {
    if ((source == null) || (source.length < 1)) {
      makeSource();
    }

    return source;
  }

  private void makeSource() {
    width = getWidth(null);
    height = getHeight(null);
    
    source = new int[width * height];
    
    //Log.log("Making new image:" + (width * height));
    
    PixelGrabber pg = new PixelGrabber(image, 0, 0, width, height, source, 0, width);
    try {
      pg.grabPixels();
    } catch (InterruptedException e) {
      Log.log(e.toString());
    }
  }

  public static final  void setToolkit(final Toolkit toolkit) {
    ImageWrapper.toolkit = toolkit;
  }

  public static final  Toolkit getToolkit() {
    return toolkit;
  }

  public final void setHeight(final int height) {
    this.height = height;
  }

  public final int getHeight() {
    return height;
  }

  public final void setWidth(final int width) {
    this.width = width;
  }

  public final int getWidth() {
    return width;
  }

  public final void setSource(final int[] source) {
    this.source = source;
  }

//  public final int[] getSource() {
//    return source;
//  }

  public int getPixelColour(int x, int y) {
    return source[x + width * y];
  }
}
