import java.awt.Graphics;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;

import javax.imageio.ImageIO;

import utilities.log.Log;
import utilities.log.LogAdaptorSystemOut;

/*
 * ToDo
 * ====
 * Fixes:
 * Add/remove final method parameters...
 * Remove unused modifiers from interfaces...
 * Move square brackets to the left in array type declarations...
 * Add braces onto if/do/while/for statements that lack them
 * Expand code substantially to handle formatting and indentation...
 * 
 * Done
 * ====
 */

public class LCDAA {
  //static String in_path_current;
  //String in_directory_current = "";

  public static void main(String args[]) {
    Log.log_adaptor = new LogAdaptorSystemOut(); // command-line logging...

    if (((args.length) > 2) || ((args.length) < 1)) {
      usage();
      return;
    }

    try {
      String in_path = args[0];
      String out_path = args[1];
      process(in_path, out_path, false);
    } catch (Exception e) {
      System.err.println("A problem has arisen:");
      System.err.println("" + e);
      e.printStackTrace();
    }
  }

  static void usage() {
    System.err.println("LCDAA:");
    System.err.println("A utility that applies LCD anti-aliasing to image files.");
    System.err.println("Usage: LCDAA <infile> <outfile>");
  }

  public static void process(String in_path, String out_path, boolean verbose) throws FileNotFoundException, IOException {
    File f_in = new File(in_path);
    String abs_in_path = f_in.getAbsolutePath();

    File f_out = new File(out_path);
    String abs_out_path = f_out.getAbsolutePath();

    Image i = processFile(abs_in_path, abs_out_path);

    saveImage(i, abs_out_path);
  }

  static Image processFile(String abs_in_path, String abs_out_path) throws FileNotFoundException, IOException {
    //LCDAA.in_path_current = in_path;
    //Log.log("" + abs_in_path);

    Image image = Toolkit.getDefaultToolkit().getImage(abs_in_path);

    waitForImageToLoad(image);

    return image;
  }

  static void saveImage(Image i, String abs_out_path) throws FileNotFoundException, IOException {
    // Log.log("" + abs_out_path);
    ImageWrapper i2 = LCDImageProcess.process(new ImageWrapper(i));

    int width = i2.getWidth();
    int height = i2.getHeight();

    BufferedImage new_image = new BufferedImage(width, height, BufferedImage.TYPE_4BYTE_ABGR);
    Graphics g = new_image.getGraphics();
    g.drawImage(i2.getImage(), 0, 0, null);

    // save captured image to PNG file
    ImageIO.write(new_image, "png", new File(abs_out_path));
  }

  private static void waitForImageToLoad(Image image) {
    do {
      try {
        Thread.sleep(10);
      } catch (InterruptedException e) {
        Log.exception(e);
      }
    } while (!Toolkit.getDefaultToolkit().prepareImage(image, -1, -1, null));
  }
}
