import java.applet.Applet;
import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Color;
import java.awt.Component;
import java.awt.FileDialog;
import java.awt.Font;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.GridLayout;
import java.awt.Image;
import java.awt.Label;
import java.awt.Panel;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.URL;

import utilities.log.Log;
import utilities.log.LogAdaptorLogWindow;

/**
 * ListClassesFrEnd - written by Tim Tyler.
 * 
 * A front end for ListClasses.<P>
 * This code has been placed in the public domain.<P>
 * You can do what you like with it.<P>
 * Note that this code comes with no warranty.<P>
 */
public class LCDAAFrEnd extends java.applet.Applet implements ActionListener {
  static String document_base;
  static String java_version;
  static String directory_separator;
  static URL base_url;

  static Panel input, output_, input1, input2, input3;
  static Panel output_1, output_2, output_3, output_4, output_5;

  static Frame frame;

  static boolean application = false;

  static final boolean development_version = false;

  static TextField input_file_textfield;
  static TextField output_file_textfield;

  static TextField key_textfield;

  static Button input_file_button;
  static Button output_file_button;

  static Button process_button;

  // constructor
  public LCDAAFrEnd() {
    SwingBugs.useDynamicLayout();

    setLayout(new BorderLayout());
    Font std_font = new Font("Helevetica", Font.BOLD, 16);

    output_5 = new Panel();
    output_5.setBackground(Color.lightGray);

    Label label_temp;

    Panel panel_basic = new Panel();
    panel_basic.setLayout(new BorderLayout());

    Panel panel_west = new Panel();
    panel_west.setLayout(new GridLayout(3, 1));
    panel_west.setBackground(Color.lightGray);

    Panel panel_east = new Panel();
    panel_east.setLayout(new GridLayout(3, 1));

    Panel panel_centre = new Panel();
    panel_centre.setLayout(new GridLayout(3, 1));
    panel_centre.setBackground(Color.lightGray);

    label_temp = new Label("Input file:", Label.RIGHT);
    label_temp.setFont(std_font);
    panel_west.add(label_temp, BorderLayout.WEST);
    input_file_textfield = new TextField("", 99);
    input_file_textfield.setFont(std_font);
    panel_centre.add(input_file_textfield, BorderLayout.CENTER);
    input_file_button = new Button("Browse");
    input_file_button.setFont(std_font);
    input_file_button.addActionListener(this);
    panel_east.add(input_file_button, BorderLayout.EAST);

    label_temp = new Label("Output file:", Label.RIGHT);
    label_temp.setFont(std_font);
    panel_west.add(label_temp, BorderLayout.WEST);
    output_file_textfield = new TextField("", 99);
    output_file_textfield.setFont(std_font);
    panel_centre.add(output_file_textfield, BorderLayout.CENTER);
    output_file_button = new Button("Browse");
    output_file_button.setFont(std_font);
    output_file_button.addActionListener(this);
    panel_east.add(output_file_button, BorderLayout.EAST);

    Panel panel1 = new Panel();
    Panel panel2 = new Panel();
    panel1.setBackground(Color.lightGray);
    panel2.setBackground(Color.lightGray);

    // panel_west.add(checkbox_verbose);
    panel_centre.add(panel1);
    panel_east.add(panel2);

    process_button = new Button("Process");
    process_button.setFont(std_font);
    process_button.addActionListener(this);
    output_5.add(process_button);

    panel_basic.add(panel_west, BorderLayout.WEST);
    panel_basic.add(panel_centre, BorderLayout.CENTER);
    panel_basic.add(panel_east, BorderLayout.EAST);

    add(panel_basic, BorderLayout.CENTER);
    add(output_5, BorderLayout.SOUTH);

    greyOutputArea();
  }

  void greyOutputArea() {}

  void enableButtons(boolean flag) {
    process_button.setEnabled(flag);
  }

  public void actionPerformed(ActionEvent ev) {
    Object  c = ev.getSource();
    
    if (c == process_button) {
      processFile();
    }

    if (c == output_file_button) {
      browseOutputFile();
    }

    if (c == input_file_button) {
      browseInputFile();
    }
  }

  private void processFile() {
    enableButtons(false);
    boolean verbose = true;
    process_button.setLabel("Processing...");
    try {
      process(verbose);
    } catch (Exception e) {
      Log.log("Error while processing files:");
      Log.exception(e);
    }

    process_button.setLabel("Process");
    enableButtons(true);
  }

  void getDocBase() {
    if (document_base == null) {
      // get Java version...
      java_version = System.getProperty("java.version");
      directory_separator = System.getProperty("file.separator");

      if (application) {
        try {
          document_base = System.getProperty("user.dir");
          base_url = new URL("file:/" + document_base + directory_separator);
        } catch (Exception e2) {
          Log.log("Error getting application base: " + e2.toString());
        }
      } else {
        try {
          // base_url = getDocumentBase(); // applet
          base_url = getCodeBase(); // applet
          document_base = base_url.toString();
          String base_url_s = base_url.toString(); // applet

          // trailing "."...?
          if (base_url_s.indexOf("/.") > 0) {
            Log.log("Warning: slash-fullstop in URL - truncating URL to avoid bug.");
            document_base = document_base.substring(0, document_base.length() - 2);
            base_url = new URL(base_url_s.substring(0, base_url_s.length() - 1));
          }
        } catch (Exception e) {
          Log.log("Error getting applet base: " + e.toString());
        }
      }
    }
  }

  static void process(boolean verbose) throws Exception {
    String input_file = input_file_textfield.getText();
    String output_file = output_file_textfield.getText();
    LCDAA.process(input_file, output_file, verbose);
  }

  static void browseInputFile() {
    FileDialog fd = new FileDialog(frame, "Select input file", FileDialog.LOAD);

    fd.setFile("input.png");

    fd.show();
    String returnedstring = fd.getFile();
    if (returnedstring != null) {
      if (!returnedstring.equals("")) {
        input_file_textfield.setText(fd.getDirectory() + returnedstring);
      }
    }
  }

  static void browseOutputFile() {
    FileDialog fd = new FileDialog(frame, "Select output file", FileDialog.SAVE);

    fd.setFile("output.png");

    fd.show();
    String returnedstring = fd.getFile();
    if (returnedstring != null) {
      if (!returnedstring.equals("")) {
        output_file_textfield.setText(fd.getDirectory() + returnedstring);
      }
    }
  }

  public static void main(String args[]) {
    application = true;

    Log.log_adaptor = new LogAdaptorLogWindow();

    LCDAAFrEnd applet = new LCDAAFrEnd();

    frame = new AppletFrame("LCDAA", (Applet) applet, 640, 150);
    Image image = frame.createImage(29, 29);
    Graphics g = image.getGraphics();
    g.setColor(Color.black);
    g.fillRect(0, 0, 30, 30);
    g.setColor(Color.yellow);
    g.drawRect(0, 0, 28, 28);

    for (int i = 1; i < 14; i++) {
      int temp = i * 14;
      g.setColor(new Color((temp << 16) | (temp ^ 0xFF)));
      g.drawRect(i, i, 28 - i - i, 28 - i - i);
    }

    frame.setIconImage(image);
  }
}
